#=======================================================================================================
#PaREn Data Sources
#
#In this module you find functions for the generation of
#synthetic distributions and loaders for natural data.
#This is a REDUCED VERSION without Translators.
#
#	Main Features:
#|	- Mixture					FUNCTION	Specify a tuple with parameters for synthetic scene generation. This is probably the function you are looking for.
#|	- QuicklyGenerateProblem	FUNCTION	If you're not interested in the nitty gritty details of a mixture, then try this simple generator.
#|  - ErrorProblem2SphericGaussians
#|                              FUNCTION    Creates a dataset with two classes where each is a spheric gaussian density with predefined Bayes error
#|  - ErrirProblem2Gaussians	FUNCTION    Creates a dataset with two classes where each has user defined gaussian density with predefined Bayes error
#|  - QuicklyGenerate2Gaussians FUNCTION    Creates a dataset with two classes where each is a user defined gaussian density with predefined distance
#|  - CreateRawEncoderProblem	FUNCTION	Creates a dataset with an encoder problem
#|  - QuicklyGenerateEncoderProblem
#|                              FUNCTION    Create various encoder problems with many defaults
#|  - 
#|
#|	Additional Functionality:
#I	- GaussianDistribution		FUNCTION	Mutlivariate gaussian with arbitrary direction, position and size
#M	- ComplexDistribution		FUNCTION	Well, some strange bodys come out of this - control it with the weirdness factor
#P	- IndependentDistribution 	FUNCTION	Square and independent distributions - they remind of boxes and rectangles. Freely positionable in multivariate feature space
#O
#R	Helper Functions
#T	- WhiteGaussian				FUNCTION	Helper. Multivariate gaussian distribution with cov=1 and center=0
#A	- ImmunizeKey				FUNCTION	Helper. Makes key objects less sensitive to different data types
#E	- ltqnorm					FUNCTION	Helper. Lower tail quantile for standard normal distribution function.
#|	- erfcinv					FUNCTION	Helper. Approximation of erfc^(-1).
#|	- HelperBellCurve			FUNCTION	Helper. Calculates class density for given normal distributions
#|	- HelperIndependent			FUNCTION	Helper. Calculates class density for given even distributions	
#v	- HelperApproxComplex		FUNCTION	Helper. Approximates class density for given complex distributions
#=======================================================================================================

#=======================================================================================================
#Usage Examples:
#
#CovarianceMatrix1 	= array( [ [ 5, 0], [0, 1] ] );				#create arbitrary covariance matrix
#CovarianceMatrix2 	= array( [ [ 3, 0.3], [0.3, 3] ] );			#create arbitrary covariance matrix
#Center1		    = array( [ [ 1 ,1 ] ] );					#specify some center vector
#Center2 			= array( [ [-1 ,-2 ] ] );
#
#Dimensions = 2;
#
#		  Class,     Type,    Samples,     Center,   OptionalMatrix
#Mix1=( (       1,   'Gauss',       5,     Center1,   CovarianceMatrix1 ), \
#		( 	    2,  'Normal',        5,	   Center2,   CovarianceMatrix2 ), \
#		(       3,    'Ind.',        5,    Center2,   eye(Dimensions) ), \
#		(       3,  'square',        5,    Center1,   eye(Dimensions) ), \
#		(       5,  'compl.',        5,    Center1,   10 ) );
#
#Result = Mixture( Mix1, Dimensions );     #Result is a Translator object
#
#Result.convert( 'LIBSVM', ( 10, 110), ( '.libSVM.vec', None, None ) );	 #-> This for example would export only MyVectors 10 to 110 to the 
#																	     #   sparse vector format of libSVM, SVMlight and SVMperf.
#
#=======================================================================================================



#=======================================================================================================
#INCLUDES
#=======================================================================================================
from math 				import *
from pylab 				import *
from numpy 				import *
from string 			import *
from scipy  			import *
from xml.dom.minidom 	import *
from time 				import *

import os
import sys
import glob
import PaREnUtility


#=======================================================================================================
#DEFAULTS
#=======================================================================================================
__author__      = "Aleksander Lodwich"
__copyright__  	= "Copyright (C) 2008 DFKI"
__version__     = "0.1"

ModuleVerbosity = 1;


#=======================================================================================================
#METHOD ClassInventory( LabelList )
#RETURNS LIST of ANYTHING
#This function finds unique class identifiers in a label list
#LabelList	: LIST. List of hashable items serving a labels
#=======================================================================================================
def ClassInventory( MyClasses ):			
	Inventory1 = {};
	
	c = len( MyClasses );
	for i in range( c ):
		Inventory1[MyClasses[i]] = 1;
		
	return Inventory1;


#=======================================================================================================
#METHOD Reproduce( Key, Number )
#RETURNS LIST of ANYTHING
#This method creates a list with Number of Keys
#Key	: ANYTHING HASHABLE. Your label, madamme
#Number	: INT. Length of list.
#=======================================================================================================
def Reproduce( Key, Number ):
	R = [];
	for i in xrange( Number ):
		R += [ Key ];
	return R;


#=======================================================================================================
#METHOD VectorCount( Matrix )
#RETURNS INT
#This method the number of vectors available in Matrix
#Matrix		: 2D Array
#=======================================================================================================
def VectorCount( Matrix ):
	return Matrix.shape[0];

#=======================================================================================================
#METHOD Dimensions( Matrix )
#RETURNS INT
#This method the number of features available in Matrix
#Matrix		: 2D Array
#=======================================================================================================
def Dimensions( Matrix ):
	return Matrix.shape[1];

#=======================================================================================================
#FUNCTION ImmunizeKey( Key )
#RETURNS STRING/DOUBLE and no integers but it also returns other hashable types
#THROWS AttributeError and HashError
#Returns a key that is not sensitive to data type
#=======================================================================================================
def ImmunizeKey( Key ):	
	if Key is None:
		raise AttributeError( 'Key is a None value' );
	
	TryHashing 	= hash( Key ); 		#if hashing is not working then exception is raised.	
	t 			= type( Key );

	if t == str or t == string_:
		return string.upper( Key );
	
	if 	t == int  or t ==  int8 or t ==  int16 or t ==  int32 or t ==  int64 or \
		t == uint or t == uint8 or t == uint16 or t == uint32 or t == uint64 or \
		t == float or t == float32:
		return double( Key );

	return Key;


#=======================================================================================================
#PRIVATE FUNCTION _FixCenterVector( CenterOrStart, Dims )
#RETURNS 2D NumPy Array
#Looks for common error with the center vector and fixes them:
#=======================================================================================================
def _FixCenterVector( CenterOrStart, Dims, Loud ):
	if CenterOrStart is None or size( CenterOrStart ) == 0:	#catch missing values
		CenterOrStart = array([zeros( Dims )]);
		WrnMsg = { "Status":'Warning', "Description":'Correcting missing values in center vector!', "MODULE":'PaREnDataSources::_FixCenterVector()' };
		Send( WrnMsg, ModuleVerbosity and Loud );
		PaREnProtocol.SubmitReport( WrnMsg );
	#end if

	CenterOrStart = PaREnUtility.FixPhysicalProperties( CenterOrStart, 2, "PaREnDataSources::Mixture()", Loud );
	CenterOrStart = PaREnUtility.Fix2DLogicalProperties( CenterOrStart, Dims, 1, "PaREnDataSources::Mixture()", Loud );

	return CenterOrStart;



#=======================================================================================================
#PRIVATE FUNCTION _FixMatrix( Matrix, Dims, Loud )
#RETURNS 2D NumPy Array
#Looks for common error with the center vector and fixes them:
#=======================================================================================================
def _FixMatrix( Matrix, Dims, Loud ):	
	
	if Matrix is None or size( Matrix ) == 0:
		Matrix = eye( Dims );
		WrnMsg = { "Status":'Warning', "Description":'Creating artificial transform matrix!', "MODULE":'PaREnDataSource::_FixMatrix()' };
		Send( WrnMsg, ModuleVerbosity and Loud );
	#end if
	
	if type( Matrix ) is not ndarray:
		Matrix = array( Matrix )

	Matrix = PaREnUtility.FixPhysicalProperties( Matrix, 2, "PaREnDataSources::Mixture()", Loud );
	Matrix = PaREnUtility.Fix2DLogicalProperties( Matrix, Dims, Dims, "PaREnDataSources::Mixture()", Loud );

	return Matrix;
		
	

#=======================================================================================================
#PRIVATE FUNCTION _HashList( List )
#RETURNS LIST
#Hashes a List and return a list of hashes. String objects are upper cased before hashing.
#Integer objects are converted into doubles before hashing.
#List = _HashList(List);
#=======================================================================================================
def _HashList( List ):	
	L =  size( List );
	Result = zeros( L ).tolist();
	for i in range( L ):
		CODE = List[i];
		t = type( CODE );
		if t == str or t == string_:
			CODE = string.upper( CODE );
			Result[i] = hash( CODE );
			continue;
		
		
		if 	t == int  or t ==  int8 or t ==  int16 or t ==  int32 or t ==  int64 or \
			t == uint or t == uint8 or t == uint16 or t == uint32 or t == uint64 or \
			t == float or t == float32:
			CODE = double( List[i] );
			Result[i] = hash( CODE );
			continue;
		Result[i] = hash( CODE ); 				#all other types with no conversion
	return Result;




#=======================================================================================================
#FUNCTION ltqnorm()
#RETURNS DOUBLE
#=======================================================================================================
def ltqnorm( p ):
	"""
	Lower tail quantile for standard normal distribution function.

	Modified from the author's original perl code (original comments follow below)
	by dfield@yahoo-inc.com.  May 3, 2004.

	This function returns an approximation of the inverse cumulative
	standard normal distribution function.  I.e., given P, it returns
	an approximation to the X satisfying P = Pr{Z <= X} where Z is a
	random variable from the standard normal distribution.

	The algorithm uses a minimax approximation by rational functions
	and the result has a relative error whose absolute value is less
	than 1.15e-9.

	Author:      Peter J. Acklam
	Time-stamp:  2000-07-19 18:26:14
	E-mail:      pjacklam@online.no
	WWW URL:     http://home.online.no/~pjacklam
	"""

	if p <= 0 or p >= 1:
		# The original perl code exits here, we'll throw an exception instead
		raise ValueError( "Argument to ltqnorm %f must be in open interval (0,1)" % p )

	# Coefficients in rational approximations.
	a = (-3.969683028665376e+01,  2.209460984245205e+02, \
		-2.759285104469687e+02,  1.383577518672690e+02, \
		-3.066479806614716e+01,  2.506628277459239e+00)
	b = (-5.447609879822406e+01,  1.615858368580409e+02, \
		-1.556989798598866e+02,  6.680131188771972e+01, \
		-1.328068155288572e+01 )
	c = (-7.784894002430293e-03, -3.223964580411365e-01, \
		-2.400758277161838e+00, -2.549732539343734e+00, \
		4.374664141464968e+00,  2.938163982698783e+00)
	d = ( 7.784695709041462e-03,  3.224671290700398e-01, \
		2.445134137142996e+00,  3.754408661907416e+00)

	# Define break-points.
	plow  = 0.02425
	phigh = 1 - plow

	# Rational approximation for lower region:
	if p < plow:
		q  = math.sqrt(-2*math.log(p))
		return (((((c[0]*q+c[1])*q+c[2])*q+c[3])*q+c[4])*q+c[5]) / \
			((((d[0]*q+d[1])*q+d[2])*q+d[3])*q+1)

	# Rational approximation for upper region:
	if phigh < p:
		q  = math.sqrt(-2*math.log(1-p))
		return -(((((c[0]*q+c[1])*q+c[2])*q+c[3])*q+c[4])*q+c[5]) / \
				((((d[0]*q+d[1])*q+d[2])*q+d[3])*q+1)

	# Rational approximation for central region:
	q = p - 0.5
	r = q*q
	return (((((a[0]*r+a[1])*r+a[2])*r+a[3])*r+a[4])*r+a[5])*q / \
		(((((b[0]*r+b[1])*r+b[2])*r+b[3])*r+b[4])*r+1)


#=======================================================================================================
#FUNCTION erfcinv()
#RETURNS DOUBLE
#z: scalar
#=======================================================================================================
def erfcinv(z):
#ERFCINV Inverse incomplete error function.
#
#   X = ERFCINV(Y) is the inverse incomplete error function for each element of
#   Y.  The incomplete inverse error functions satisfies y = erfc(x), for 0 <=
#   y <= 2 and -inf <= x <= inf.
#
#   See also ERFC, ERF.
#
#   Author:      Peter J. Acklam
#   Time-stamp:  2003-04-13 11:45:24 +0200
#   E-mail:      pjacklam@online.no
#   URL:         http://home.online.no/~pjacklam
#
#	Array extension: Aleksander Lodwich
	Dimensions 	= shape( z );							#get original dimensions
	FieldCount	= size( z );							#get number of elements in array
	z			= z.reshape( FieldCount ); 				#Change indexing of the field to be 1D linear
	Result 		= zeros( FieldCount );					#create a straight 1D array long enough to accomodate z
	
	for i in range( FieldCount ):						#for all elements in z
		Result[i] = -ltqnorm( z[i] / 2 ) / sqrt(2);		#transform uniform distribution to normal distribution

	Result 		= Result.reshape( Dimensions );			#turn 1D into whatever-D back again.
	return Result;										



#=======================================================================================================
#FUNCTION WhiteGaussian()
#RETURNS Translator Object
#=======================================================================================================
def WhiteGaussian( Count, Dimensions ):
	return sqrt( 2 ) * erfcinv( rand( Count, Dimensions ) * 2 );


#=======================================================================================================
#FUNCTION GaussianDistribution()
#RETURNS Translator Object
#Count:			No. of samples to create
#Dimensions:	No. of dimensions of the problem
#mu:			Center of normal distribution
#epsilon:		Target covariance of normal distribution
#=======================================================================================================
def GaussianDistribution( Count, Dimensions, mu, epsilon ):
	mu					= _FixCenterVector( mu, Dimensions, 0 );
	epsilon				= _FixMatrix( epsilon, Dimensions, 0 );
	EigenDescription 	= eig( epsilon );												#Calculate destortion directions
	Transformation		= transpose( EigenDescription[1] ) * sqrt(EigenDescription[0]);	#make transformation from white gauss to target gauss
	Temp 				= WhiteGaussian( Count, Dimensions );							#create white normal distribution
	NormalDistribution  = dot(Transformation, \
							transpose( Temp ) ) + transpose(mu);						#apply transformation
	Result 				= transpose(NormalDistribution);								#put data into the Translator Container!
	return Result;																		#return vector population
	

#=======================================================================================================
#FUNCTION ComplexDistribution()
#RETURNS Translator Object
#Count:			No. of samples to create
#Dimensions:	No. of dimensions of the problem
#StartVec:		Center/Start vector of complex distribution
#Weirdness:		0: Spot, 1: 'gaussian', 10:really weird
#=======================================================================================================
def ComplexDistribution( Count, Dimensions, StartVec, Weirdness ):
	Vector    = _FixCenterVector( StartVec, Dimensions, 0 );
	Implosion	= 1 - exp( -Weirdness );
	Vectors 	= zeros( (Count, Dimensions ) );

	for i in range( Count ):
		Vector = ( Vector + ( 2 * rand( 1, Dimensions ) ) - 1 )* Implosion;
		Vectors[i,:] = Vector;
	#end for

	return Vectors;	#return vector population


#=======================================================================================================
#FUNCTION IndependentDistribution()
#RETURNS Translator Object
#Count:			No. of samples to create
#Dimensions:	No. of dimensions of the problem
#StartVec:		Center/Start vector of complex distribution
#Trans.Matrix:	Transformation Matrix (Optional)
#=======================================================================================================
def IndependentDistribution( Count, Dimensions, StartVec, TransformationMatrix ):
	StartVec    = _FixCenterVector( StartVec, Dimensions, 0 );
	TransformationMatrix = _FixMatrix( TransformationMatrix, Dimensions, 0 );
	Vectors 	= rand( Dimensions, Count ) - 0.5;
	Result		= transpose( dot( TransformationMatrix, Vectors ) + transpose( StartVec ) );
	return Result;	#return vector population



#=======================================================================================================
#HELPER FUNCTION HelperBellCurve( Vector, Center, Epsilon)
#RETURNS 2D NumPy Array (ndarray)
#Calculates probability (scalar) at this vector, when bell curve is placed at center and is given the inverted cov. matrix.
#Vector:			Multivariate points (horizontal vectors arranged vertically in 2D ndarray)
#Center:			Center/Start vector of normal distribution (horiz. vector 1D ndarray) 
#Epsilon:			Covariance matrix (2D ndarray, height and width equal to width of >>Vector<<
#=======================================================================================================
def HelperBellCurve( Vector, Center, Epsilon ):
	if type( Epsilon ) != ndarray:								#this is true for 1D
		Epsilon = array( [[Epsilon]] );

	if type( Center ) != ndarray:								#this is true for 1D
		Center = array( [[Center]] );
		
	InverseEpsilon 	= linalg.inv( Epsilon );					#We do this for performance reasons. Dont calculate this too often
	SqrtDeterminant = sqrt( abs( linalg.det( Epsilon ) ) );		#We do this for performance reasons. Dont calculate this too often

	j = Vector - Center;	
	d = len( Center );
	B = transpose( exp(-0.5 * sum( transpose(j)*dot( InverseEpsilon, transpose(j) ), 0 ) ) / ( (2*pi)**(d/2.0) * SqrtDeterminant) );
	return B;


#=======================================================================================================
#HELPER FUNCTION FastBellCurve( Vector, Center, Epsilon)
#RETURNS 2D NumPy Array (ndarray)
#Calculates probability (scalar) at this vector, when bell curve is placed at center and is given the inverted cov. matrix.
#Vector:			Multivariate points (horizontal vectors arranged vertically in 2D ndarray)
#Center:			Center/Start vector of normal distribution (horiz. vector) 
#Epsilon:			Covariance matrix
#=======================================================================================================
def FastBellCurve( Vector, Center, InverseEpsilon, SqrtDeterminant ):
	j = Vector - Center;
	d = len( Center );
	B = transpose( exp(-0.5 * sum( transpose(j)*dot( InverseEpsilon, transpose(j) ), 0 ) ) / ( (2*pi)**(d/2.0) * SqrtDeterminant) );
	return B;


#=======================================================================================================
#HELPER FUNCTION HelperIndependent( Vector, Center, Transform )
#RETURNS 2D NumPy Array (ndarray)
#Calculates probability (scalar) at this vector, when box is placed at center and is transformed
#Vector:			Multivariate points (horizontal vectors arranged vertically)
#Center:			Center/Start vector of normal distribution (vertical vector)
#Epsilon:			Covariance matrix
#=======================================================================================================
def HelperIndependent( Vectors, CenterOrStart, Transform ):
	#First you have to calculate the volume of a box transformed with these parameters.
	#Because the integral of the box must be normed to one, the probability for any point to occur within
	#the box is 1/Volume.
	#There are two cases that have to discerned: a point is wihtin the box where it is 1/n probable 
	#to belong to the class or it is outside the box where it's probability is zero.
	#This is done by calculating a reverse transform matrix and by applying it to the vectors.
	#The retransformed vectors must have all |coordinates|<=.5 for being considered within the box.
	if not isinstance( Vectors, ndarray ):
		Vectors = array([[Vectors]]);
	
	if len( shape( Vectors ) ) == 1:
		Vectors = array( [Vectors] ); 						#add one dimension
	
	Dimensions			= size( [ CenterOrStart ] );
	N					= size( Vectors, 0 );
	Probabilities		= zeros( ( N, 1 ) );

	#We will calculate the probability for a point being within the original box. Now we need to check all vectors for doing so, indeed.
	#First we create the augmented forward transformation and then we revert the transformation to get the backward matrix.
	#We appy the backward matrix and investigate the dim ranges for all of the vectors.
	
	Forward 			= concatenate( (concatenate( (Transform, transpose( CenterOrStart ) ), 1 ), concatenate( ( [zeros( Dimensions )],  [[1]] ), 1 ) ), 0 );
	Backward 			= inverse( Forward );

	AugmentedVectors 	= concatenate( (transpose( Vectors ), ones( (1,N) )), 0 );
	OriginalVectors  	= maximum( sign( -abs( dot( Backward, AugmentedVectors )[0:Dimensions,:] ) + 0.5 ), 0 );	
	QualifiedVectors 	= maximum( sum( OriginalVectors, 0 ) - Dimensions + 1, 0 );
	Probability 		= 1.0/sum( QualifiedVectors );
	Probabilities 		= QualifiedVectors * Probability;
	
	return Probabilities;
	#end FUNCTION
	

#=======================================================================================================
#HELPER FUNCTION HelperComplex( Vector, Classes, GenerateForClass )
#RETURNS 2D NumPy Array (ndarray)
#Estimates a probability (scalar) for every supplied vector in >>Vector<<
#through density estimation on all >>Vector<<s
#Vector:			Multivariate points (horizontal vectors arranged vertically)
#Classes:   		Classification of every Vector there's in.
#GenerateForClass:	Hashable class identifier
#=======================================================================================================
def HelperComplex( Vectors, Classes, GenerateForClass ):
	HashedClasses 	= _HashList( Classes[:] ); 						#the [:] serves as the .copy() function
	HashedKey		= _HashList( [GenerateForClass] );
	N				= size( Vectors, 0 );
	SelectedVectors = int8(equal( HashedClasses, HashedKey ));
	Selected 		= sum( SelectedVectors );
	
	if Selected == 0:
		return array([[]]);
	
	Indices			= transpose( ( array( [range( N )] ) ) + 1);
	Indices			= unique( Indices * SelectedVectors ) - 1;
	Slice			= Vectors[Indices[1:],:];
	Distances		= zeros( (Selected,1) );
	Probabilities	= zeros( N );

	for i in range( N ):				#Now, after the class representatives were extracted, calculate probabilities for all of the vectors
		for j in range( Selected ):
			Distances[j]	= norm( Vectors[i,:] - Vectors[j,:] );
		#end for - all representatives
		Distances.sort(0);
		Variance 		= std( Distances );
		TakeClosest 	= argmin( abs( Distances-Variance ) ) ;
		Probabilities[i]= double(TakeClosest) / double(Selected);
		
	#end for - all vectors
	return Probabilities;



#=======================================================================================================
#FUNCTION Mixture( Description, Dims )
#RETURNS Translator Object
#
#Description is a list or tuple of tuples of the type:
#[ ( Class, 'Gaussian', Count, CenterVector, CovarianceMatrix), ...
#  ( Class, 'Independent', Count, Center, TransformMatrix), ... 
#  ( Class, 'Complex', Count, Center, Weirdness ) ]
#
#The values for Count, Center, Weirdness etc. can be set to None. In this case defaults are used!
#The names are used in fault tolerant way, so that you can ignore case, include whitespace and abbreviate names:
#'GAUSSIAN','Gaussian Distribution', '  gauss  ', 'ga', 'Normal Distribution', 'NORMAL', 'NO', 'no' -> Gaussian Distribution
#'Complex', 'COMPLEX', 'comp. distribution', 'co', '   complex ', 'weird stuff', 'WEIRDO' -> Complex Distribution
#'independent', 'square', 'ind. distro.', 'square method...' -> Independent Distribution
#
#Dims: 	Integer number > 0 for the number of dimensions this mixture should be created for. Because
#		this number must be the same across all of the distributions it's not part of the descriptions.
#Normalize	: 0/1. Default is 1 (ON). If you need direct output of the density function just turn to zero (OFF)
#In contrast to the simpler cousins of this method generates a probability matrix for ground truth.
#=======================================================================================================
def Mixture( Descriptions, Dims, Normalize=1 ):
	#SECTION INDEX:
	#SECTION A: GUARD
	#SECTION B: Generating all of the asked for vectors
	#SECTION C: Calculating Probability Matrix		
	#SECTION D: Normalizing Probability Matrix		
	StartTime = time();
	Quiet	= 0;
	Loud	= 1;
	
	#SECTION A: GUARD
	if Dims < 1:
		raise Exception, 'Zero and negative dimensions do not work!'
	
	#behind this strange try structure hides the real hard attempt to make it work anyhow, without calling errors.
	try:
		if len( Descriptions[0] ) < 5:
				raise Exception;
	except:
		Descriptions = [Descriptions];
		try:
			if len( Descriptions[0] ) < 5:
					raise Exception;
		except:
			raise Exception, 'Bad format for Description. Make sure it is a [[Class,Type,Count,Center,Option,...]]'
	#end of the real hard attempt to make it work.

		
	#SECTION B: Generating all of the asked for vectors
	DensityCount 	= len( Descriptions );
	MyVectors 		= None;	
	MyLabels		= [];
	MyProbabilities = None;
	MyClassIndex	= {};
	
	for i in range( DensityCount ):											#Do for all descriptions:

		Description 			= Descriptions[i];							#pick it:
			
		if not PaREnUtility.IsHashable( Description[0] ):
			raise Exception, 'First element must be hashable'
			#end if
		#end if
		
		if not type( Description[1] ) is str:
			raise Exception, 'Second element must be string describing the density.'
			#end if
		#end if		

		if not PaREnUtility.IsNumeric( Description[2] ):
			raise Exception, 'Third element must be an integer.'
			#end if
		#end if

			
		if len( Description ) < 4:											#and check for parameter count
			raise Exception, 'Not enough parameters to describe a density. Skipping...'
			#end if
		#end if
																			#extract properties:															
		try:
			GenerateForClass	= ImmunizeKey( Description[0] );			#can be anything hashable
		except:
			continue;														#if something is wrong with the class key then skip description

		TypeOfDensity			= strip( upper( Description[1] ) );			#GenerateForClass must be string -> STRING
		NumberOfExamples		= Description[2];
		CenterOrStart			= Description[3];
		
		if NumberOfExamples <= 0:											#catch senseless values
			raise Exception, 'Created a distribution with no samples!'
		#end if
		
		if len( Description ) > 4:							#do we have optional parameters?
			OptionalParameter	= Description[4];
		#end if
		
		CenterOrStart = _FixCenterVector( CenterOrStart, Dims, Loud );

		
		#Execute task for normal distribution:
		if TypeOfDensity.find( 'GA' ) == 0 or TypeOfDensity.find( 'NO' ) == 0: 
			StartTime2 = time();
			OptionalParameter = _FixMatrix( OptionalParameter, Dims, Loud );
			Gauss = GaussianDistribution( NumberOfExamples, Dims, CenterOrStart, OptionalParameter );
			MyLabels += Reproduce( GenerateForClass, NumberOfExamples );
			MyVectors = AddVertical( MyVectors, Gauss );
		#end if
		
		#Execute task fro independent "square" distribution:
		if TypeOfDensity.find( 'IN' ) == 0 or TypeOfDensity.find( 'SQ' ) == 0:
			OptionalParameter = _FixMatrix( OptionalParameter, Dims, Loud );
			Independent = IndependentDistribution( NumberOfExamples, Dims, CenterOrStart, OptionalParameter ); #Optional Parameter is a Dim x Dim transform matrix
			MyLabels += Reproduce( GenerateForClass, NumberOfExamples );
			MyVectors = AddVertical( MyVectors, Independent );
		#end if
		
		#Execute task for weird distributions. If there are too many point for a given OptinalParameter, then disitrbution can tend towards normal.
		if TypeOfDensity.find( 'CO' ) == 0 or TypeOfDensity.find( 'WE' ) == 0:
			if not isscalar( OptionalParameter ):
				OptionalParameter = 1;
			#end if not scalar
			Weird = ComplexDistribution( NumberOfExamples, Dims, CenterOrStart, OptionalParameter );
			MyLabels += Reproduce( GenerateForClass, NumberOfExamples );
			MyVectors = AddVertical( MyVectors, Weird );
		#end if
	#end for (Descriptions)


	#SECTION C: Calculating Probability Matrix		
		
	LocalClassIndex = ClassInventory( MyLabels ); 	#searches for class id's in MyClasses, not in MyClassIndex which we want to build now	
	ClassIDs		= LocalClassIndex.keys();   	#This dictionary is coming from ClassInventory(), so dont use _SortedColumns()
	n				= len( ClassIDs );
	c 				= VectorCount( MyVectors );
		
	for i in range( n ):
		Description = Descriptions[i];				#pick it:
		MyClassIndex[ClassIDs[i]] = i; 				#this is assigns arbitrary columns to the class. We can do this now, because it is not assigned, yet.
	#end for
	
	MyProbabilities = zeros( ( c, n ) );

	#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	#What happens here?
	#
	#          
	#			<-VectorCount->
	#       +-------------------------+  ^
	#       | Class 1 Probability     <----------------------+
	#       | Class 2 Probability     |  |                   |
	#       |     "MyProbabilities    |  ClassCount          |
	#       |          Matrix"        |	 |                   |
	#       | Class ClassCount        |  |                   |
	#       +-------------------------+  V                   |
	#                                                   +----o---+
	#                                                   |   ADD  | please look for EXPLAIN REMARK
	#			<-VectorCount->							+-o-o--o-+
	#       +-------------------------+  ^                | |  |
	#Class 3|                         |  |                | |  |
	#Class 1|                         O-------------------+ |  |
	#Class 1|                         O---------------------+  |
	#Class 2|                         |  |                     |
	#Class 3|                         |  |                     |
	#Class 2|                         |  DensityCount          |
	#Class 2|                         |  |                     |
	#Class 1|                         O------------------------+
	# ...   |     Virtual Density     |  |
	# ...   |          Matrix         |	 |
	# ...   |                         |  |
	#       +-------------------------+  V
	#
	#,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	

	for DensityIndex in range( DensityCount ):						#for all known density functions...inspect any vector....

		Description 			= Descriptions[DensityIndex];		#pick it:
		
		try:														
			GenerateForClass 	= ImmunizeKey( Description[0] );		 #here some obscure keys could have been put in..
			ClassColumn			= MyClassIndex[GenerateForClass]; #this will become neccessary when some sample value were set to zero
		except:
			continue;
		
		TypeOfDensity			= strip( upper( Description[1] ) );	#GenerateForClass must be string -> STRING
		NumberOfExamples		= Description[2];
		CenterOrStart			= Description[3];
		OptionalParameter		= None;
		
		if len( Description ) > 4:									#do we have optional parameters?
			OptionalParameter	= Description[4];
		#end if
		FunctionType  = 1;											#assume gaussian as default (1:gauss, 2:independent, 3:complex)
		
		CenterOrStart = _FixCenterVector( CenterOrStart, Dims, Quiet );
		
		#Execute task fro independent "square" distribution:
		if TypeOfDensity.find( 'IN' ) == 0 or TypeOfDensity.find( 'SQ' ) == 0:
			FunctionType = 2;
		#end if
		
		#Execute task for weird distributions. If there are too many point for a given OptinalParameter, then disitrbution can tend towards normal.
		if TypeOfDensity.find( 'CO' ) == 0 or TypeOfDensity.find( 'WE' ) == 0:
			FunctionType = 3;
		#end if

		#these functions calculate rows of virtual matrix! ---+ HelperBellCurve
		#                                                     | HelperIndependent
		#                                                     | HelperComplex
		#                                                     V

		if FunctionType == 1: 
			OptionalParameter = _FixMatrix( OptionalParameter, Dims, Quiet );
			MyProbabilities[:,ClassColumn] += HelperBellCurve( MyVectors, CenterOrStart, OptionalParameter ); 		#EXPLAIN REMARK
		#end if
		
		if FunctionType == 2:
			OptionalParameter = _FixMatrix( OptionalParameter, Dims, Quiet );
			MyProbabilities[:,ClassColumn] += HelperIndependent( MyVectors, CenterOrStart, OptionalParameter );  		#EXPLAIN REMARK
		#end if 
		
		if FunctionType == 3:
			MyProbabilities[:,ClassColumn] += HelperComplex( MyVectors, MyLabels, GenerateForClass ); 		#EXPLAIN REMARK
		#end if 
	#end for
	
	#SECTION D: Normalizing Probability Matrix		
	if Normalize:
		Sums = maximum(transpose(array([sum( MyProbabilities, 1 )])), 0.0000001);
		MyProbabilities = MyProbabilities / Sums;

	return MyVectors, MyLabels, MyProbabilities, ClassIDs;


#end def Mixture();


#=======================================================================================================
#FUNCTION QuicklyGenerateProblem( Vector, Classes, GenerateForClass )
#RETURNS TRANSLATOR
#Quickly generates a gaussian problem from few parameters:
#Classes:   		How many classes should be in the data?
#Dimensions:		How many features does the data have?
#Samples:			What is the total amount of samples you'd like to get?
#Easy:				Large numbers mean easy problems
#Spacing:			FLOAT 0..1. Tells how evenly spaced the classes are: 1.0 evenly spaced. 0.0 random parts.
#=======================================================================================================
def QuicklyGenerateProblem( Classes=2, Dimensions=2, Samples=100, Easy=3, Spacing=0.5 ):
	#SECTION A: GUARD INPUT
	Spacing 	= PaREnUtility.InRange( 0.0, Spacing, 1.0 );  #must be between 0..1
	Easy 		= PaREnUtility.InRange( 0.0, Easy, 100.0 );
	Classes 	= int( Classes );
	Dimensions 	= int( Dimensions );
	Samples		= int( Samples );
	
	#SECTION B: CREATE MIXTURE DESCRIPTIONS
	Descriptions = [];
	RelativeSizes = AddHorizontal( array([0]), ones( Classes ) * Spacing + rand( Classes ) * ( 1.0 - Spacing ) );
	RelativeSizes /= sum( RelativeSizes );	
	Positions = int32( cumsum( RelativeSizes ) * Samples );
	Positions[-1] = Samples; #just make sure it really is exactly what we need
	for Class in xrange( 1, Classes+1 ):
		ClassSamples = Positions[Class] - Positions[Class-1] ;
		
		Center = rand( 1, Dimensions ) * sqrt( Classes * Easy );
		CovMatrix = eye( Dimensions ) * rand( Dimensions, Dimensions );
		
		Description = [ Class, "gauss", ClassSamples, Center, CovMatrix ];
		Descriptions += [ Description ];
	#end for
	
	#SECTION C: GENERATE MIXTURE
	Result = Mixture( Descriptions, Dimensions );
	return Result;
			

#=======================================================================================================
#FUNCTION QuicklyGenerate2Gaussians( Dimensions, Samples, Distance )
#RETURNS TRANSLATOR
#Quickly generates a gaussian problem from few parameters:
#Cov1:				Covariance Matrix 1
#Cov2:				Covariance Matrix 2
#Samples:			What is the total amount of samples you'd like to get?
#Distance:			Euclidian Distance
#=======================================================================================================
def QuicklyGenerate2Gaussians( Cov1, Cov2, Samples=100, Distance=2.0 ):
	#SECTION A: GUARD INPUT
	if var( list( shape( Cov1 ) )  + list( shape( Cov2 ) ) ) > 0.0:
		ErrMsg = { "Status":'Error', "Description":'Covariance matrices must be square and of same size!', "MODULE":'PaREnDataSources::QuicklyGenerate2Gaussians()' };
		Send( ErrMsg, ModuleVerbosity )
		PaREnProtocol.SubmitReport( ErrMsg )
		return Translator.Empty()
	
	Dimensions 	= int( shape( Cov1 )[0] );
	Samples		= int( Samples );
	
	#SECTION B: CREATE MIXTURE DESCRIPTIONS
	Descriptions = [];
		
	Center = zeros( ( 1, Dimensions ) );
	
	Description = [ 'A', "gauss", Samples, Center, Cov1 ];
	Descriptions += [ Description ];


	Center = ones( (1, Dimensions) ) * Distance / sqrt( Dimensions );
	CovMatrix = eye( Dimensions ) * rand( Dimensions, Dimensions );
	
	Description = [ 'B', "gauss", Samples, Center, Cov2 ];
	Descriptions += [ Description ];

	#SECTION C: GENERATE MIXTURE
	return Mixture( Descriptions, Dimensions );
			



#=======================================================================================================
#FUNCTION ErrorProblem2SphericGaussians( Vector, Classes, GenerateForClass )
#RETURNS 2D Array, List, 2D Array, List, FLOAT
#Quickly generates a gaussian problem from few parameters:
#Error:		   		Float, how much error should be possible to make
#Dimensions:		How many features does the data have?
#Samples:			What is the total amount of samples you'd like to get?
#Tolearance:		FLOAT. Small number that defines how long the refinement will take place
#=======================================================================================================
def ErrorProblem2SphericGaussians( Error, Dimensions=2, Samples=100, Tolerance=0.0001 ):
	Dimensions 	= int( Dimensions );
	Samples		= int( Samples );
	Error		= PaREnUtility.InRange( 0.0, Error, 0.5 );
	Right		= 100.0;
	CovMatrix 	= eye( Dimensions );
	ZeroCenter 	= array( [ zeros( Dimensions ) ] );
	Left 		= 0.0;
	Distance	= 0.5 * Right;
	AvgError	= inf;
	Overload	= 2;
	DescriptionZero = [ 0, "gauss", Samples, ZeroCenter, CovMatrix ];
	Whity		= GaussianDistribution( Samples*Overload, Dimensions, ZeroCenter, CovMatrix );	
	
	Probabilities = zeros( ( Samples*2, 2 ) );
	Selected1	= range( Samples*Overload )
	Selected2	= range( Samples*Overload )
	LastError	= inf;
	Vector		= array([ ones( Dimensions ) ]);
	ion()
	
	while abs( AvgError - Error ) > Tolerance and 2*(Right - Left) > Tolerance:
		Center 		= Vector * Distance;
		AvgError	= 0.0;	
		Try			= 1;
		
		random.shuffle( Selected1 )
		random.shuffle( Selected2 )
		Whity1			= Whity[Selected1[:Samples],:][:];
		Whity2			= Whity[Selected2[:Samples],:][:];
		Whity2 += Center;
		MyVectors = AddVertical( Whity1, Whity2 );		
		Probabilities[:,0] = HelperBellCurve( MyVectors, ZeroCenter, CovMatrix );
		Probabilities[:,1] = HelperBellCurve( MyVectors, Center, CovMatrix );
		Sums = maximum(transpose(array([sum( Probabilities, 1 )])), 0.0000001);
		Probabilities = Probabilities / Sums;		
		AvgError = mean( Probabilities.min( 1 ) )

		#AvgError /= rhat;
		Delta = abs( AvgError - Error )
		#print "|Avg-Error|=", Delta, " Distance=", norm( ones( Dimensions ) * Distance ), "    EstimatedError=", AvgError
		if AvgError < Error:		
			Right = Left + ( Right - Left ) * 0.5;
			#print "too small"
		else:
			Left = Right - ( Right - Left ) * 0.5;
			#print "too large"
			
		Distance = Left + ( Right - Left ) * 0.5;
	#until resulterror is good
	
	MyClasses = Reproduce( 'A', VectorCount( Whity1 ) ) + Reproduce( 'B', VectorCount( Whity2 ) );
#	sys.stdin.readline()
	return MyVectors, MyClasses, Probabilities, [ 'A', 'B' ], norm( ones( Dimensions ) * Distance )



#=======================================================================================================
#FUNCTION ErrorProblem2Gaussians( Vector, Classes, GenerateForClass )
#RETURNS 2D Array, List, 2D Array, List, FLOAT
#Quickly generates a gaussian problem from few parameters:
#Error:		   		FLOAT. how much error should be possible to make
#Cov1:				2D Array. Covariance matrix for density 1
#Cov2:				2D Array. Covariance matrix for density 2
#Dimensions:		INT. How many features does the data have?
#Samples:			INT. What is the total amount of samples you'd like to get?
#Tolearance:		FLOAT. Small number that defines how long the refinement will take place
#=======================================================================================================
def ErrorProblem2Gaussians( Error, Cov1, Cov2, Dimensions=2, Samples=100, Tolerance=0.001 ):
	#SECTION A: GUARD INPUT
	Dimensions 	= int( Dimensions );
	Samples		= int( Samples );
	Error		= PaREnUtility.InRange( 0.0, Error, 0.5 );
	Right		= 100;
	ZeroCenter 	= array( [ zeros( Dimensions ) ] );
	Left 		= 0.0;
	Distance	= 0.5 * Right;
	AvgError	= inf;
	R 			= [ sqrt(eig( Cov1 )[0]).max()*3, sqrt(eig( Cov2 )[0]).max()*3 ];

	DescriptionZero = [ "A", "gauss", Samples, ZeroCenter, Cov1 ];
	#DEBUG START
	while abs( AvgError - Error ) > Tolerance and 2*(Right - Left) > Tolerance:
		Descriptions 	= [];
		#Sphere 1
		Center = array( [ ones( Dimensions ) * Distance ] );
			
		Description = [ "B", "gauss", Samples, Center, Cov2 ];
		Descriptions += [ Description ];
		
		#Sphere 2
			
		Descriptions += [ DescriptionZero ];
		
		Vectors, Labels, Probabilities, ClassIDs = Mixture( Descriptions, Dimensions, 1 );
		AvgError = mean( Probabilities.sum( 1 ) - Probabilities.max( 1 ) );
		#print "Ps=", Probabilities
		#print "x=", Distance, "    Distance=", norm( ones( Dimensions ) * Distance ), "    AvgError=", AvgError, "   |Avg-Error|=", abs( AvgError - Error )
		
		if AvgError < Error:		
			Right = Left + ( Right - Left ) * 0.5;
		else:
			Left = Right - ( Right - Left ) * 0.5;
			
		Distance = Left + ( Right - Left ) * 0.5;
	#until resulterror is good
	
	return Vectors, Labels, Probabilities, ClassIDs, norm( ones( Dimensions ) * Distance );


#=======================================================================================================
#FUNCTION Unary
#RETURNS 1d Array
#Creates a vector, where one dimension is set to one and others are set to zero
#Example Unary( 1, 4 ):  array( [ 0, 1, 0, 0 ] )
#i		:	INTEGER. Which dimension should be 1
#n		:   INTEGER. How many dimensions should be in the vector.
#=======================================================================================================
def Unary( i, n ):
	result = zeros(n);
	result[i] = 1;
	return result;


#=======================================================================================================
#FUNCTION CreateRawEncodeProblem
#RETURNS 2d Array, 2dArray
#Creates two 2d arrays with input and target vectors for training of an encoder problem. The encoder
#problem is used in auto encoding learning algorithms like artificial neural networks.
#This function can add noisy samples to the input or output side or both sides and beyond that it
#can add noise inputs in the input mode.
#Dims		: INTEGER. Number of dimensions per vector
#Samples	: INTEGER. Number of vectors to generate
#Mode		: STRING. Describes whether there should be noise added to input, output or both
#             Examples: "in", "out", "input and output", "Output", "Input/Output" ...
#Noise 		: FLOAT. A coefficient for a random signal that is normally scaled 0..1
#NoiseUnits	: INTEGER. How many noise input units should be added. Only works in input mode.
#=======================================================================================================
def CreateRawEncoderProblem( Dims, Samples=100, Mode="input", Noise=0.0, NoiseUnits=0 ):
	""" Create input and target vectors for a MNM problem with gaussian noise
		according to the Dims size.
		where = {'input', 'output', 'inputoutput'}
		Noise : strength of the noise (float) >= 0.0
		NoiseUnits : add NoiseUnits new noisy inputs in database (INT>=0)
	"""
	where  = 0;
	INPUT  = 1;
	OUTPUT = 2;
	INOUT  = 3;
	
	if Mode.upper().find( 'IN' ) >= 0:
		where += INPUT;
		
	if Mode.upper().find( 'OUT' ) >= 0:
		where += OUTPUT;
	
	if where < 1:
		ErrMsg = { "Status":'Error', "Description":'Where parameter has no known state.', "MODULE":'PaREnDataSources::CreateRawEncodeProblem()' };
		Send( ErrMsg, ModuleVerbosity );
		SubmitReport( ErrMsg );
		return [ None, None ];
	
	if where == INPUT:
		database_i = rand( Samples, Dims + NoiseUnits ) * Noise;
		database_t = zeros( ( Samples, Dims)  );
			
	if where == OUTPUT:
		database_t = rand( Samples, Dims + NoiseUnits ) * Noise;
		database_i = zeros( ( Samples, Dims ) );

	if where == INOUT:
		database_i = rand( Samples, Dims + NoiseUnits ) * Noise;
		database_t = rand( Samples, Dims + NoiseUnits ) * Noise;

	for i in xrange( Samples ):
		x = i % Dims;
		database_i[i][x] += 1.0;
		database_t[i][x] += 1.0;

	return database_i, database_t;


#=======================================================================================================
#UNCONDITIONAL EXECUTION (MAIN)
#UNIT TESTING
#=======================================================================================================

if __name__ == "__main__":
	print "PERFORMING SELFTEST"
	print "=========================================================================================="
	
	print "TESTING ClassInventory"
	Inventory = ClassInventory( [ "A", 1, 1, 1, "A", "B", 2, 2, 1, "B" ] );
	TestCases = { 'A':1, 1:1, 2:1, 'B':1, 'C':0, 'D':0, 3:0 };
	for TestCase in TestCases.items():
		if Inventory.get( TestCase[0], 0 ) != TestCase[1]:
			raise Exception, "ClassInventory() not working!"

	print "TESTING AddVertical"
	Connected = AddVertical( rand( 10, 5 ), rand( 20, 5 ), None, rand( 10 , 5 ) );
	if shape( Connected ) != ( 40, 5 ):
		raise Exception, "AddVertical() is not working!"

	print "TESTING AddHorizontal"
	Connected = AddHorizontal( rand( 10, 5 ), rand( 10, 5 ), None, rand( 10 , 5 ) );
	if shape( Connected ) != ( 10, 15 ):
		raise Exception, "AddHorizontal() is not working!"

	print "TESTING Reproduce"
	x = Reproduce( "A", 5 );
	if x != [ "A", "A", "A", "A", "A" ]:
		raise Exception, "ReproduceKey() is not working!"
	
	print "TESTING VectorCount"
	if VectorCount( rand(10,4) ) != 10:
		raise Exception, "VectorCount() is not working!"
	
	print "TESTING Dimensions"
	if Dimensions( rand(10,4) ) != 4:
		raise Exception, "Dimensions() is not working!"
	
	print "TESTING _FixCenterVector"
	x = _FixCenterVector( [ 0,0,0 ], 4, 0 );
	if shape( x ) != ( 1,4 ):
		raise Exception, "_FixCenterVector() is not working!1"
	
	x = _FixCenterVector( [ [0,0,0] ], 3, 0 );
	if shape( x ) != ( 1,3 ):
		raise Exception, "_FixCenterVector() is not working!2"

	x = _FixCenterVector( array([[ [0,0,0] ]]), 3, 0 );
	if shape( x ) != ( 1,3 ):
		raise Exception, "_FixCenterVector() is not working!3"

	x = _FixCenterVector( array([[ [0],[0],[0] ]]), 3, 0 );
	if shape( x ) != ( 1,3 ):
		raise Exception, "_FixCenterVector() is not working!4"

	print "TESTING _FixMatrix"
	x = _FixMatrix( [ [[1,2,3],[1,2,3],[4,5,6],[8,8,8]]], 3, 0 );
	if ( x != array([[1,2,3],[1,2,3],[4,5,6]] ) ).any():
		raise Exception, "_FixMatrix() is not working!"

	print "TESTING _HashList"
	if _HashList( [ 1,type(2),3] ) != [ 1, 7506240, 3 ]:
		raise Exception, "_HashList() is not working!"
	
	print "TESTING ltqnorm"
	if abs( ltqnorm( 0.1 ) + 1.28155156414 ) > 0.00001:
		raise Exception, "ltqnorm error 1 "+ str( ltqnorm( 0.1 ))

	if abs( ltqnorm( 0.2 ) + 0.841621232727 ) > 0.00001:
		raise Exception, "ltqnorm error 2 "+ str( ltqnorm( 0.2 ))

	if abs( ltqnorm( 0.3 ) + 0.524400513279 ) > 0.00001:
		raise Exception, "ltqnorm error 3 "+ str( ltqnorm( 0.3 ))

	if abs( ltqnorm( 0.99 ) - 2.32634787439 ) > 0.00001:
		raise Exception, "ltqnorm error 4 "+ str( ltqnorm( 0.99 ))

	print "TESTING erfcinv"
	GotValues = erfcinv( array( [ 0.1, 0.3, 0.7, 0.9 ] ) )
	ShouldValues = array([ 1.16308715, 0.73286908, 0.27246271, 0.08885599])
	if ( abs(  GotValues - ShouldValues ) > 0.00001 ).any():
		raise Exception, "erfcinv error 1 "+ str( GotValues )

	print "TESTING HelperBellCurve"
	x = array( [ [ -1, 1, 0 ], [ 0, 0, 0 ], [ 1, 1, 1 ], [ 0, -1, 1 ] ] );
	y = array( [ 1, 1, 1 ] )
	x = HelperBellCurve( x, y, eye( 3 ) );
	if ( abs(  x - array( [ 0.00521188, 0.01416735, 0.06349364, 0.00521188] ) ) > 0.00001 ).any():
		raise Exception, "HelperBellCurve " + str( x )

	print "TESTING FastBellCurve"
	x = array( [ [ -1, 1, 0 ], [ 0, 0, 0 ], [ 1, 1, 1 ], [ 0, -1, 1 ] ] );
	y = array( [ 1, 1, 1 ] )
	x = FastBellCurve( x, y, eye( 3 ), sqrt( linalg.det( eye( 3 ) ) ) ) ;
	if ( abs(  x - array( [ 0.00521188, 0.01416735, 0.06349364, 0.00521188] ) ) > 0.00001 ).any():
		raise Exception, "FastBellCurve " + str( x )


	print "TESTING WhiteGaussian"
	x = WhiteGaussian( 100, 5 );
	if shape( x ) != ( 100, 5 ):
		raise Exception, "WhiteGaussian() is not working!"
	
	print "TESTING GaussianDistribution"
	x = GaussianDistribution( 10000, 4, ones(4)*10, eye( 4 )*2 );
	if shape( x ) != ( 10000, 4 ):
		print "Shape(x)=", shape(x), "but should be (10000,4)"
		raise Exception, "GaussianDistribution() is not working!"
	
	if ( abs( mean( x, 0 ) - ones( 4 ) * 10 ) > 0.03 ).any():
		print abs( mean( x, 0 ) - ones( 4 ) * 10 )
		raise Exception, "GaussianDistribution() is not working! This can happen if random is unlucky. Try again until it passes."
	
	print "TESTING HelperComplex"
	x = int32( rand( 100 ) * 3 );
	x = HelperComplex( rand( 100, 2 ), x, 1 );
	if shape( x ) != ( 100, ):
		print x
		raise Exception, "HelperComplex() is not working!"

	print "TESTING ComplexDistribution"
	x = ComplexDistribution( 1000, 4, ones( 4 ) * 10, 10 );
	if shape( x ) != ( 1000, 4 ):
		print "Shape(x)=", shape(x), "but should be (1000,4)"
		raise Exception, "ComplexDistribution() is not working!"
	
	print "TESTING IndependentDistribution"
	x = IndependentDistribution( 1000, 4, zeros( 4 ), eye( 4 ) );
	if shape( x ) != ( 1000, 4 ):
		print "Shape(x)=", shape(x), "but should be (1000,4)"
		raise Exception, "IndependentDistribution() is not working!"
	
	if ( abs( mean( x, 0 ) ) > 0.02 ).any():
		print mean( x, 0 )
		raise Exception, "IndependentDistribution() is not working!"
	
	print "TESTING Mixture"
	Description = [ [ "A", "Gauss", 100, ones( 4 ), eye( 4 ) ], [ "B", "independent", 100, -ones( 4 ), eye( 4 ) ], ( "C", "weird stuff", 100, array( [ 1, 0, 1, 0 ] ), 10 ) ];
	Vecs, Labels, Probs, ClassIDs = Mixture( Description, 4 );
	if shape( Vecs ) != (300,4 ):
		raise Exception, "Mixture not working CODE 1"
	
	if len( Labels ) != 300:
		raise Exception, "Mixture not working CODE 2"
	
	if shape( Probs ) != ( 300, 3 ):
		raise Exception, "Mixture not working CODE 3"
	
	if len( ClassIDs ) != 3 or "A" not in ClassIDs or "B" not in ClassIDs or "C" not in ClassIDs:
		raise Exception, "Mixture not working CODE 4"
	
	print "TESTING QuicklyGenerateProblem"
	for C in [ 2, 3, 5 ]:
		for D in [ 2, 4, 8 ]:
			for S in [ 100, 500, 1000 ]:
				for E in [ 1, 10 ]:
					for L in [ 0.1, 0.5, 1.0 ]:
						Vecs, Labels, Probs, ClassIDs = QuicklyGenerateProblem( C, D, S, E, L );
						if len( ClassInventory( Labels ) ) != C:
							raise Exception, "QuicklyGenerateProblem CODE 1" + str( [ C, D, S, E, L ] )
						
						if shape( Vecs ) != ( S, D ):
							raise Exception, "QuicklyGenerateProblem CODE 2"+ str( [ C, D, S, E, L ] )
						
						if shape( Probs ) != ( S, C ):
							raise Exception, "QuicklyGenerateProblem CODE 3"+ str( [ C, D, S, E, L ] )

						if len( ClassIDs ) != C:
							raise Exception, "QuicklyGenerateProblem CODE 4"+ str( [ C, D, S, E, L ] )

	print "TESTING QuicklyGenerate2Gaussians"
	Vecs, Labels, Probs, ClassIDs = QuicklyGenerate2Gaussians( eye( 3 ) * 5, eye( 3 ) * 2, 100, 20 );
	if shape( Vecs ) != (200, 3 ):
		print shape( Vecs )
		raise Exception, "QuicklyGenerate2Gaussians not working CODE 1"
	
	if len( Labels ) != 200:
		print Labels
		raise Exception, "QuicklyGenerate2Gaussians not working CODE 2"
	
	if shape( Probs ) != ( 200, 2 ):
		print shape( Probs )
		raise Exception, "QuicklyGenerate2Gaussians not working CODE 3"
	
	if len( ClassIDs ) != 2 or "A" not in ClassIDs or "B" not in ClassIDs:
		print ClassIDs
		raise Exception, "QuicklyGenerate2Gaussians not working CODE 4"

	print "TESTING ErrorProblem2Gaussians and ErrorProblem2SphericGaussians"
	x = 0;
	for D in [ 2, 4, 8 ]:
		for S in [ 100, 1000, 10000 ]:
			Vecs2, Labels2, Probs2, ClassIDs2, Dist2 = ErrorProblem2SphericGaussians( 0.2, D, S );
			Vecs, Labels, Probs, ClassIDs, Dist = ErrorProblem2Gaussians( 0.2, eye( D ), eye( D ), D, S );
			print D, S, Dist, Dist2

			if abs( Dist - Dist2 ) > 0.05:
				print "Illegal discrepancy between ErrorProblem2SphericGaussians and ErrorProblem2Gaussians!"
				x += 1;
				if x > 4:
					raise Exception, "The discrepncies seem not to be due to random numbers!"

			if abs( Dist - Dist2 ) > 0.15:
				raise Exception, "Illegal discrepancy between ErrorProblem2SphericGaussians and ErrorProblem2Gaussians too serious!"

			if shape( Vecs ) != (2*S, D ):
				print Vecs
				print shape( Vecs ), "expected", (2*S, D )
				raise Exception, "ErrorProblem2Gaussians not working CODE 1"
			
			if len( Labels ) != 2*S:
				print Labels
				raise Exception, "ErrorProblem2Gaussians not working CODE 2"
			
			if shape( Probs ) != ( 2*S, 2 ):
				print Probs
				print shape( Probs ), "expected", (2*S, 2 )
				raise Exception, "ErrorProblem2Gaussians not working CODE 3"
			
			if len( ClassIDs ) != 2 or "A" not in ClassIDs or "B" not in ClassIDs:
				print ClassIDs
				print ClassIDs
				raise Exception, "ErrorProblem2Gaussians not working CODE 4"


			if shape( Vecs2 ) != (2*S, D ):
				print shape( Vecs2 )
				raise Exception, "ErrorProblem2SphericGaussians not working CODE 1"
			
			if len( Labels2 ) != 2*S:
				print Labels2
				raise Exception, "ErrorProblem2SphericGaussians not working CODE 2"
			
			if shape( Probs2 ) != ( 2*S, 2 ):
				print shape( Probs2 )
				raise Exception, "ErrorProblem2SphericGaussians not working CODE 3"
			
			if len( ClassIDs2 ) != 2 or "A" not in ClassIDs or "B" not in ClassIDs:
				print ClassIDs2
				raise Exception, "ErrorProblem2SphericGaussians not working CODE 4"
			
	print "TESTING CreateRawEncoderProblem"
	I, O = CreateRawEncoderProblem( 3, 20, Noise=0.05, Mode="Input" );	
	if abs(I-O).max() > 0.05:
		raise Exception, "Strange noise function?!"

	I, O = CreateRawEncoderProblem( 3, 20, Noise=0.05, Mode="Output" );	
	if abs(I-O).max() > 0.05:
		raise Exception, "Strange noise function?!"

	I, O = CreateRawEncoderProblem( 3, 20, Noise=0.05, Mode="Input and Output" );	
	if abs(I-O).max() > 0.1:
		raise Exception, "Strange noise function?!"

	I, O = CreateRawEncoderProblem( 3, 20, Noise=0.05, Mode="Input", NoiseUnits=2 );	
	if I.shape[1] != 5 or O.shape[1] != 3:
		raise Exception, "Problem with additional noise units in input!"

	I, O = CreateRawEncoderProblem( 3, 20, Noise=0.05, Mode="Output", NoiseUnits=2 );	
	if I.shape[1] != 3 or O.shape[1] != 5:
		raise Exception, "Problem with additional noise units in output!"

	I, O = CreateRawEncoderProblem( 3, 20, Noise=0.05, Mode="Input Out", NoiseUnits=2 );	
	if I.shape[1] != 5 or O.shape[1] != 5:
		raise Exception, "Problem with additional noise units in mixed mode!"

	
	print "TESTING QuicklyGenerateEncode"
	print "=========================================================================================="
	print "SELFTEST FINISHED."
	print "if the test has arrived here, then the module suffices minimal operational quality."
#=======================================================================================================
#END OF FILE
#=======================================================================================================
